From: Dmitry Torokhov <dtor_core@ameritech.net>
To: Vojtech Pavlik <vojtech@suse.cz>
Cc: linux-kernel@vger.kernel.org
Subject: [patch 02/24] uinput: use completions
Date: Mon, 25 Jul 2005 00:34:51 -0500 [thread overview]
Message-ID: <20050725054530.798684000.dtor_core@ameritech.net> (raw)
In-Reply-To: 20050725053449.483098000.dtor_core@ameritech.net
[-- Attachment #1: uinput-use-completion.patch --]
[-- Type: text/plain, Size: 6151 bytes --]
Input: uinput - use completions instead of events and manual
wakeups in force feedback code.
Signed-off-by: Dmitry Torokhov <dtor@mail.ru>
---
drivers/input/misc/uinput.c | 81 +++++++++++++++++++++++---------------------
include/linux/uinput.h | 5 +-
2 files changed, 45 insertions(+), 41 deletions(-)
Index: work/include/linux/uinput.h
===================================================================
--- work.orig/include/linux/uinput.h
+++ work/include/linux/uinput.h
@@ -42,8 +42,7 @@ struct uinput_request {
int code; /* UI_FF_UPLOAD, UI_FF_ERASE */
int retval;
- wait_queue_head_t waitq;
- int completed;
+ struct completion done;
union {
int effect_id;
@@ -62,7 +61,7 @@ struct uinput_device {
struct uinput_request *requests[UINPUT_NUM_REQUESTS];
wait_queue_head_t requests_waitq;
- struct semaphore requests_sem;
+ spinlock_t requests_lock;
};
#endif /* __KERNEL__ */
next prev parent reply other threads:[~2005-07-25 5:54 UTC|newest]
Thread overview: 25+ messages / expand[flat|nested] mbox.gz Atom feed top
2005-07-25 5:34 [patch 00/24] Input patches for 2.6.13 Dmitry Torokhov
2005-07-25 5:34 ` [patch 01/24] uinput: formatting, cleanup Dmitry Torokhov
2005-07-25 5:34 ` Dmitry Torokhov [this message]
2005-07-25 5:34 ` [patch 03/24] serio: add modalias Dmitry Torokhov
2005-07-25 5:34 ` [patch 04/24] Acecad: small cleanup Dmitry Torokhov
2005-07-25 5:34 ` [patch 05/24] Add usb_to_input_id Dmitry Torokhov
2005-07-25 5:34 ` [patch 06/24] sonypi: make sure input_work is not running when unloading Dmitry Torokhov
2005-07-25 5:34 ` [patch 07/24] input: rearrange procfs code Dmitry Torokhov
2005-07-25 5:34 ` [patch 08/24] input: make name, phys and uniq const char Dmitry Torokhov
2005-07-25 5:34 ` [patch 09/24] input.c section fix Dmitry Torokhov
2005-07-25 5:34 ` [patch 10/24] serio_raw: link misc device and serio port Dmitry Torokhov
2005-07-25 5:35 ` [patch 11/24] serio_raw - fix Kconfig help Dmitry Torokhov
2005-07-25 5:35 ` [patch 12/24] i8042 - add Alienware Sentia to NOMUX blacklist Dmitry Torokhov
2005-07-25 5:35 ` [patch 13/24] i8042 - add Fujitsu T3010 " Dmitry Torokhov
2005-07-25 5:35 ` [patch 14/24] synaptics - limit rate on Toshiba Dynabooks Dmitry Torokhov
2005-07-25 5:35 ` [patch 15/24] ALPS: Fix resume for DualPoints Dmitry Torokhov
2005-07-25 5:35 ` [patch 16/24] ALPS: fix enabling tapping mode Dmitry Torokhov
2005-07-25 5:35 ` [patch 17/24] HID: Add a quirk for Aashima gamepad Dmitry Torokhov
2005-07-25 5:35 ` [patch 18/24] joydev - remove MSECS macro Dmitry Torokhov
2005-07-25 5:35 ` [patch 19/24] elo - fix help in Kconfig Dmitry Torokhov
2005-07-25 5:35 ` [patch 20/24] HID - only report events coming from interrupts to hiddev Dmitry Torokhov
2005-07-25 5:35 ` [patch 21/24] psmouse: wheel mice always have middle button Dmitry Torokhov
2005-07-25 5:35 ` [patch 22/24] i8042 - dont use negation in i8042_command() Dmitry Torokhov
2005-07-25 5:35 ` [patch 23/24] Input - check keycodesize when adjusting keymaps Dmitry Torokhov
2005-07-25 5:35 ` [patch 24/24] Synaptics - fix setting packet size on passthrough port Dmitry Torokhov
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20050725054530.798684000.dtor_core@ameritech.net \
--to=dtor_core@ameritech.net \
--cc=linux-kernel@vger.kernel.org \
--cc=vojtech@suse.cz \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox
Index: work/drivers/input/misc/uinput.c
===================================================================
--- work.orig/drivers/input/misc/uinput.c
+++ work/drivers/input/misc/uinput.c
@@ -53,24 +53,23 @@ static int uinput_dev_event(struct input
return 0;
}
-static int uinput_request_alloc_id(struct input_dev *dev, struct uinput_request *request)
+static int uinput_request_alloc_id(struct uinput_device *udev, struct uinput_request *request)
{
/* Atomically allocate an ID for the given request. Returns 0 on success. */
- struct uinput_device *udev = dev->private;
int id;
int err = -1;
- down(&udev->requests_sem);
+ spin_lock(&udev->requests_lock);
for (id = 0; id < UINPUT_NUM_REQUESTS; id++)
if (!udev->requests[id]) {
- udev->requests[id] = request;
request->id = id;
+ udev->requests[id] = request;
err = 0;
break;
}
- up(&udev->requests_sem);
+ spin_unlock(&udev->requests_lock);
return err;
}
@@ -79,70 +78,78 @@ static struct uinput_request* uinput_req
/* Find an input request, by ID. Returns NULL if the ID isn't valid. */
if (id >= UINPUT_NUM_REQUESTS || id < 0)
return NULL;
- if (udev->requests[id]->completed)
- return NULL;
return udev->requests[id];
}
-static void uinput_request_init(struct input_dev *dev, struct uinput_request *request, int code)
+static inline int uinput_request_reserve_slot(struct uinput_device *udev, struct uinput_request *request)
{
- struct uinput_device *udev = dev->private;
+ /* Allocate slot. If none are available right away, wait. */
+ return wait_event_interruptible(udev->requests_waitq,
+ !uinput_request_alloc_id(udev, request));
+}
- memset(request, 0, sizeof(struct uinput_request));
- request->code = code;
- init_waitqueue_head(&request->waitq);
+static void uinput_request_done(struct uinput_device *udev, struct uinput_request *request)
+{
+ complete(&request->done);
- /* Allocate an ID. If none are available right away, wait. */
- request->retval = wait_event_interruptible(udev->requests_waitq,
- !uinput_request_alloc_id(dev, request));
+ /* Mark slot as available */
+ udev->requests[request->id] = NULL;
+ wake_up_interruptible(&udev->requests_waitq);
}
-static void uinput_request_submit(struct input_dev *dev, struct uinput_request *request)
+static int uinput_request_submit(struct input_dev *dev, struct uinput_request *request)
{
- struct uinput_device *udev = dev->private;
int retval;
/* Tell our userspace app about this new request by queueing an input event */
uinput_dev_event(dev, EV_UINPUT, request->code, request->id);
/* Wait for the request to complete */
- retval = wait_event_interruptible(request->waitq, request->completed);
- if (retval)
- request->retval = retval;
+ retval = wait_for_completion_interruptible(&request->done);
+ if (!retval)
+ retval = request->retval;
- /* Release this request's ID, let others know it's available */
- udev->requests[request->id] = NULL;
- wake_up_interruptible(&udev->requests_waitq);
+ return retval;
}
static int uinput_dev_upload_effect(struct input_dev *dev, struct ff_effect *effect)
{
struct uinput_request request;
+ int retval;
if (!test_bit(EV_FF, dev->evbit))
return -ENOSYS;
- uinput_request_init(dev, &request, UI_FF_UPLOAD);
- if (request.retval)
- return request.retval;
+ request.id = -1;
+ init_completion(&request.done);
+ request.code = UI_FF_UPLOAD;
request.u.effect = effect;
- uinput_request_submit(dev, &request);
- return request.retval;
+
+ retval = uinput_request_reserve_slot(dev->private, &request);
+ if (!retval)
+ retval = uinput_request_submit(dev, &request);
+
+ return retval;
}
static int uinput_dev_erase_effect(struct input_dev *dev, int effect_id)
{
struct uinput_request request;
+ int retval;
if (!test_bit(EV_FF, dev->evbit))
return -ENOSYS;
- uinput_request_init(dev, &request, UI_FF_ERASE);
- if (request.retval)
- return request.retval;
+ request.id = -1;
+ init_completion(&request.done);
+ request.code = UI_FF_ERASE;
request.u.effect_id = effect_id;
- uinput_request_submit(dev, &request);
- return request.retval;
+
+ retval = uinput_request_reserve_slot(dev->private, &request);
+ if (!retval)
+ retval = uinput_request_submit(dev, &request);
+
+ return retval;
}
static int uinput_create_device(struct uinput_device *udev)
@@ -189,7 +196,7 @@ static int uinput_open(struct inode *ino
if (!newdev)
goto error;
memset(newdev, 0, sizeof(struct uinput_device));
- init_MUTEX(&newdev->requests_sem);
+ spin_lock_init(&newdev->requests_lock);
init_waitqueue_head(&newdev->requests_waitq);
newinput = kmalloc(sizeof(struct input_dev), GFP_KERNEL);
@@ -551,8 +558,7 @@ static int uinput_ioctl(struct inode *in
}
req->retval = ff_up.retval;
memcpy(req->u.effect, &ff_up.effect, sizeof(struct ff_effect));
- req->completed = 1;
- wake_up_interruptible(&req->waitq);
+ uinput_request_done(udev, req);
break;
case UI_END_FF_ERASE:
@@ -566,8 +572,7 @@ static int uinput_ioctl(struct inode *in
break;
}
req->retval = ff_erase.retval;
- req->completed = 1;
- wake_up_interruptible(&req->waitq);
+ uinput_request_done(udev, req);
break;
default: